Attribute VB_Name = "modEditor"
'-----------------------------------------------------------
' Program Name: Rodent's Revenge 2000 Level Editor
' Authors:      James Emmrich, Alexander Popov
'
'    http://www.emu8086.com
'
' Module:       modFunctions.bas
'
' Purpose: Holds all functions that will be called by the
'          main form.
'-----------------------------------------------------------

Option Explicit

'-----------------------------------------------------------
'Program Varables
Public Const EditorTitle As String = "Rodent's Revenge 2000 Level Editor"

Public Const FIELD_WIDTH As Integer = 22  'cols
Public Const FIELD_HEIGHT As Integer = 22 'rows

'Sprite Constants
Public Const TILE_WALL As Integer = 0
Public Const TILE_BUSH As Integer = 1
Public Const TILE_GROUND As Integer = 2
Public Mouse1Tile As Integer              'Holds the currently selected tile for mouse button 1
Public Mouse2Tile As Integer              'Holds the currently selected tile for mouse button 2

Public LevelArray() As Integer            'Holds the level the user is creating
Public FieldDirty As Boolean              'Holds true if user mades changes and has not saved


'-----------------------------------------------------------
Function Editor_InitGameField()
    Dim i As Integer, j As Integer
    
    ' #021012 +1 because we count from zero...
    
    'Size the map area...
    frmEditor.picPlayingField.Width = TILE_WIDTH * (FIELD_WIDTH + 1) + 2   '+2 because of picture box border
    frmEditor.picPlayingField.Height = TILE_HEIGHT * (FIELD_HEIGHT + 1) + 2 '+2 because of picture box border
    
    'Size the arrat to fit the game field...
    ReDim LevelArray(0 To FIELD_HEIGHT, 0 To FIELD_WIDTH) As Integer
   
    'Fill the map with the ground tile...
    For i = 0 To FIELD_HEIGHT
        For j = 0 To FIELD_WIDTH
            frmEditor.picPlayingField.PaintPicture frmEditor.imgSprite(TILE_GROUND).Picture, j * TILE_WIDTH, i * TILE_HEIGHT
        Next
    Next
    
    'Place a wall around top and bottom...
    For i = 0 To FIELD_WIDTH
        frmEditor.picPlayingField.PaintPicture frmEditor.imgSprite(TILE_WALL).Picture, i * TILE_WIDTH, 0
        frmEditor.picPlayingField.PaintPicture frmEditor.imgSprite(TILE_WALL).Picture, i * TILE_WIDTH, FIELD_HEIGHT * TILE_HEIGHT
    Next
    
    'Place a wall around left and right sides...
    For i = 0 To FIELD_HEIGHT
        frmEditor.picPlayingField.PaintPicture frmEditor.imgSprite(TILE_WALL).Picture, 0, i * TILE_HEIGHT
        frmEditor.picPlayingField.PaintPicture frmEditor.imgSprite(TILE_WALL).Picture, FIELD_WIDTH * TILE_WIDTH, i * TILE_HEIGHT
    Next

    'Select the default tile for stamp (Bushes)...
    Mouse1Tile = TILE_BUSH
    Mouse2Tile = TILE_GROUND
    frmEditor.imgMouse1.Picture = frmEditor.imgSprite(Mouse1Tile).Picture
    frmEditor.imgMouse2.Picture = frmEditor.imgSprite(Mouse2Tile).Picture
    
    FieldDirty = False
End Function

'-----------------------------------------------------------
Function Editor_DrawGameTable()
'This function draws the games table to start the level.
    
    Dim i As Integer, j As Integer, ColPos As Integer, RowPos As Integer
    
    'Transfer a new level to CurrentLevel...
    ReDim CurrentLevel(0, NumTableCols, NumTableRows) As Integer
    i = 0
    j = 0
    
    For j = 0 To NumTableRows 'Row loop...
        i = 0
        CurrentLevel(0, j, i) = Levels(LevelNumber, j, i)
        
        For i = 0 To NumTableCols 'Column loop...
            CurrentLevel(0, j, i) = Levels(LevelNumber, j, i)
        Next
    Next

    'Clear table.
    frmEditor.picPlayingField.Picture = LoadPicture()
    
    i = 0
    j = 0
    ColPos = 0
    RowPos = 0
    
    For j = 0 To NumTableRows 'Row loop...
        For i = 0 To NumTableCols 'Column loop...
            'Draw Image...
            frmEditor.picPlayingField.PaintPicture frmEditor.imgSprite(Val(CurrentLevel(0, j, i))).Picture, ColPos, RowPos, TILE_WIDTH, TILE_HEIGHT
            ColPos = ColPos + TILE_HEIGHT
        Next
        
        ColPos = 0
        RowPos = RowPos + TILE_WIDTH
    Next
    
' #021012
''    'Next we set the rodents position which is easy...
''    'in the center.
''    frmEditor.picPlayingField.PaintPicture frmEditor.imgSprite(RODENT_SPRITE).Picture, (NumTableCols / 2) * TILE_WIDTH, (NumTableRows / 2) * TILE_HEIGHT, TILE_WIDTH, TILE_HEIGHT
''
''    'Make sure we place the rodent in the CurrentLevel Array...
''    CurrentLevel(0, NumTableRows / 2, NumTableCols / 2) = RODENT_SPRITE

    ' #021012
    frmEditor.Caption = EditorTitle & " - [" & LevelNames(LevelNumber) & "]"
    
End Function

