Attribute VB_Name = "modFunctions"
'-----------------------------------------------------------
' Program Name: Rodent's Revenge 2000
' Authors:      James Emmrich, Alexander Popov
'
'    http://www.emu8086.com
'
' Module:       modFunctions.bas
'
' Purpose: Holds all functions that will be called by the
'          game loop. This does not count form events.
'-----------------------------------------------------------

Option Explicit

'-----------------------------------------------------------
'Program Varables
Public Const GAME_TITLE As String = "Rodent's Revenge 2000"
Public Const JAMES_WEBSITE As String = "http://www26.brinkster.com/jemmrich"
Public Const ALEX_WEBSITE As String = "http://www.emu8086.com"

'Level File Variables & Constants
Public NumOfLevels As Integer    'Holds the number of levels contained in the level file.
Public Const NumTableCols As Integer = 22 'Holds the number of columns in the levels.
Public Const NumTableRows As Integer = 22 'Holds the number of rows in the levels.
Public Const TILE_WIDTH As Integer = 16
Public Const TILE_HEIGHT As Integer = 16
Public Const START_NEW_LEVEL_AFTER_WAVE As Integer = 5 ' #021012

'Level Arrays
Public LevelNames() As String    'Holds the names of the levels.
Public Levels() As Integer       'Holds the level data eg. bush Positions.
Public CurrentLevel() As Integer 'Holds the current levels data which can be manipulated.
Public CatsInWave As Integer     'Holds the number of cats in the current wave.

'Game Variables
Public LevelNumber As Integer    'Holds the current level of the player.
Public IsGame As Boolean         'Holds true if a game is active.
Public GamePaused As Boolean     'Holds true if the user has paused the game.

'Rodent Variables
Public RodentLivesLeft As Integer 'Holds the number of lives the rodent has left (zero based!).
Public Score As Integer          'Hold the players score!
Public RodentCol As Integer      'Holds the column number of the rodent.
Public RodentRow As Integer      'Holds the row number of the rodent.
Public DieSceneActive As Boolean 'Holds true if the rodent is dieing and is used to
                                 'prevent user from moving the mouse as the animation
                                 'is in progress
Public Const CHEESE_EATING_POINTS As Integer = 50 'The number of points player gets for eating cheese!




'Cat Variables
Public Type CatStruct
    Row As Integer               'Row position NOT pixel location.
    Col As Integer               'Column position NOT pixel location.
    State As CatState            'State: Sleeping(0), Active(1), or Not Playing(2).
End Type
Public Enum CatState
    CAT_SLEEPING = 0
    CAT_ACTIVE = 1
    CAT_NOTPLAYING = 2
End Enum
Public Cats(0 To 9) As CatStruct 'The game can have a max of 10 cats at one time.

'Sprite Constants
Public Const WALL_SPRITE = 0
Public Const BUSH_SPRITE = 1
Public Const GROUND_SPRITE = 2
Public Const RODENT_SPRITE = 3
Public Const CAT_SPRITE = 4
Public Const CATSLEEPING_SPRITE = 5
Public Const CHEESE_SPRITE = 6
Public Const TRAP_SPRITE = 7
Public Const BALL_SPRITE = 8

'API
Public Declare Function ShellExecute Lib "shell32.dll" Alias "ShellExecuteA" (ByVal hwnd As Long, ByVal lpOperation As String, ByVal lpFile As String, ByVal lpParameters As String, ByVal lpDirectory As String, ByVal nShowCmd As Long) As Long
Public Const SW_SHOWNORMAL = 1
'-----------------------------------------------------------

'-----------------------------------------------------------
Function InitializeVariables(intLevel As Integer, Optional bResetScore As Boolean = True)
'Initializes all game variables and sets CurrentLevel array.
    
    Dim i As Integer, j As Integer
    
    'Transfer a new level to CurrentLevel...
    ReDim CurrentLevel(0, NumTableCols, NumTableRows) As Integer
    i = 0
    j = 0
    
    For j = 0 To NumTableRows 'Row loop...
        i = 0
        CurrentLevel(0, j, i) = Levels(intLevel, j, i)
        
        For i = 0 To NumTableCols 'Column loop...
            CurrentLevel(0, j, i) = Levels(intLevel, j, i)
        Next
    Next
    
    'Show the levels name in the title...
    frmGameTable.Caption = GAME_TITLE & " - " & LevelNames(intLevel)
    
    CatsInWave = 1      'Start with one cat
    RodentLivesLeft = 3 'Rodent has 3 lives
    
    ' #021012
    If bResetScore Then
        Score = 0
    End If
    
'' #021012  removed:
''    If intLevel <> 0 Then
''        CatsInWave = CatsInWave + 1      'Start with one cat
''    Else
''        CatsInWave = 1
''    End If
    
    For i = frmGameTable.imgLife.LBound To frmGameTable.imgLife.UBound
        frmGameTable.imgLife(i).Visible = True
    Next
    
    DieSceneActive = False
    GamePaused = False
    
End Function

'-----------------------------------------------------------
'
' altered: #021012
'
Function LoadLevelData()
On Error GoTo err_load_ldata
'Loads the levels data into a simple to use array.
    
    Dim InputData As String, InputData1 As String, InputData2 As String
    Dim i As Integer, j As Integer, iLevelNum As Integer
    
    Dim fNum As Integer
    Dim s As String
    
    fNum = FreeFile
    
    ' set general data about levels:

        ' calculate the Number of levels in game,
        ' by checking the number of level files:
        NumOfLevels = 0
        s = NumOfLevels
        
        Do While FileExists(Add_BackSlash(App.Path) & "Levels\" & make_min_len(s, 5, "0") & ".rodent_level")
            NumOfLevels = NumOfLevels + 1
            s = NumOfLevels
        Loop
        
        ' we count from zero, this var keeps the upper index,
        ' actual number of levels - 1 :
        NumOfLevels = NumOfLevels - 1
        
        ' =============================================
        
        'Redeclare level arrays to proper size...
        ReDim LevelNames(NumOfLevels) As String
        ReDim Levels(NumOfLevels, NumTableCols, NumTableRows) As Integer
        
    
   
    'Fill the level arrays with level data...
        
    For iLevelNum = 0 To NumOfLevels
           
       s = iLevelNum
    
       Open Add_BackSlash(App.Path) & "Levels\" & make_min_len(s, 5, "0") & ".rodent_level" For Input As fNum
    
        i = 0
        j = 0
        
         '   Do Until EOF(fNum)
                For j = 0 To NumTableRows 'Row loop...
                    For i = 0 To NumTableCols 'Column loop...
                        Input #fNum, InputData
                        Levels(iLevelNum, j, i) = InputData
                    Next
                Next
                
                Input #fNum, InputData1 'Read the levels name...
                LevelNames(iLevelNum) = InputData1
        '        iLevelNum = iLevelNum + 1
               
        '    Loop
        Close #fNum
    
    Next iLevelNum
    
    LevelNumber = 0 'Set the current level.
    IsGame = False  'There is no game active yet!
    frmGameTable.lblStatus.Visible = False 'Don't show the game over message!

    Exit Function
err_load_ldata:
    MsgBox "Error loading levels!" & vbNewLine & _
    "Please check out that ""Levels"" folder is extracted correctly," & vbNewLine & _
    "when unzipping make sure ""Use folder names"" is turned on."
    End
    
End Function

'-----------------------------------------------------------
Function DrawGameTable()
'This function draws the games table to start the level.
    
    Dim i As Integer, j As Integer, ColPos As Integer, RowPos As Integer
    
    
    'Clear table.
    frmGameTable.picGameTable.Picture = LoadPicture()
    
    i = 0
    j = 0
    ColPos = 0
    RowPos = 0
    
    For j = 0 To NumTableRows 'Row loop...
        For i = 0 To NumTableCols 'Column loop...
            'Draw Image...
            ' #021012 frmGameTable.picGameTable.PaintPicture frmGameTable.imgSprite(Val(CurrentLevel(LevelNumber, j, i))).Picture, ColPos, RowPos, TILE_WIDTH, TILE_HEIGHT
            frmGameTable.picGameTable.PaintPicture frmGameTable.imgSprite(CurrentLevel(0, j, i)).Picture, ColPos, RowPos, TILE_WIDTH, TILE_HEIGHT
            ColPos = ColPos + TILE_HEIGHT
            
            ' #021012
            If CurrentLevel(0, j, i) = RODENT_SPRITE Then
                RodentRow = j
                RodentCol = i
            End If
            
        Next
        
        ColPos = 0
        RowPos = RowPos + TILE_WIDTH
    Next
    
    ' #021012
    ' required only if not set by the map:
    If RodentRow = 0 Or RodentCol = 0 Then
        'Next we set the rodents position which is easy...
        'in the center.
        RodentRow = NumTableRows / 2
        RodentCol = NumTableCols / 2
        frmGameTable.picGameTable.PaintPicture frmGameTable.imgSprite(RODENT_SPRITE).Picture, RodentCol * TILE_WIDTH, RodentRow * TILE_HEIGHT, TILE_WIDTH, TILE_HEIGHT
    
        'Make sure we place the rodent in the CurrentLevel Array...
        CurrentLevel(0, RodentRow, RodentCol) = RODENT_SPRITE
    End If
    
    
    'Place one cat on the table...
    With Cats(0)
        .Col = 2
        .Row = 2
        .State = CAT_ACTIVE
    End With
    frmGameTable.picGameTable.PaintPicture frmGameTable.imgSprite(CAT_SPRITE).Picture, Cats(0).Col * TILE_WIDTH, Cats(0).Row * TILE_HEIGHT, TILE_WIDTH, TILE_HEIGHT
    
    'Make sure we place the cat in the CurrentLevel Array...
    CurrentLevel(0, Cats(0).Row, Cats(0).Col) = CAT_SPRITE
    
    'Set all cats to not playing...
    For i = 1 To UBound(Cats)
        With Cats(i)
            .Col = 1
            .Row = 1
            .State = CAT_NOTPLAYING
        End With
    Next
    
End Function

'-----------------------------------------------------------
Function KeyboardRespond(intKeyCode As Integer)
'Acts on the players keystrokes and moves anything that
'the rodent can push such as bushes for example.
    
    Dim intValueOne As Integer, intValueTwo  As Integer, IntStepVal As Integer
    Dim i As Integer
    
    'Don't let the rodent move if he is dieing...
    If DieSceneActive = True Or GamePaused = True Then
        Exit Function
    End If
    
    'What direction is the rodent going to move in? Once we know
    'setup the values the will help us out later in this function.
    Select Case intKeyCode
        Case vbKeyLeft
            intValueOne = RodentCol - 1
            intValueTwo = 0 'End at the left wall
            IntStepVal = -1 'Backward (Right to Left)
        Case vbKeyRight
            intValueOne = RodentCol + 1
            intValueTwo = NumTableCols 'End at the right wall
            IntStepVal = 1 'Forwards (Left to Right)
        Case vbKeyUp
            intValueOne = RodentRow - 1
            intValueTwo = 0 'End at the top wall
            IntStepVal = -1 'Up (Down to Up)
        Case vbKeyDown
            intValueOne = RodentRow + 1
            intValueTwo = NumTableRows 'End at the bottom wall
            IntStepVal = 1 'Down (Up to Down)
    End Select
        
    'Is there ground on the same line in the direction the
    'rodent wants to move? Lets start at the block in front
    'of the rodents current position. This process depends
    'on either left and right or up and down... thus we check
    'direction again.
    If intKeyCode = vbKeyLeft Or intKeyCode = vbKeyRight Then
        'Then we should check the ground right in front (left or right) of the
        'rodent for ground...
        If CurrentLevel(0, RodentRow, intValueOne) = GROUND_SPRITE Then
            'Well because there is ground right in front of us we can skip
            'the rest of this function and place the rodent there BUT first
            'erase the rodents previous location.
            Call MoveRodent(RodentRow, intValueOne, RodentRow, RodentCol)
            
            'Remember new position
            RodentCol = intValueOne
            Exit Function
            
        ElseIf CurrentLevel(0, RodentRow, intValueOne) = CHEESE_SPRITE Then
            'Hey the player ate some cheese, give some points!!
            Call UpdateScore(CHEESE_EATING_POINTS)
            
            Call MoveRodent(RodentRow, intValueOne, RodentRow, RodentCol)
            
            'Remember new position
            RodentCol = intValueOne
            Exit Function
            
        ElseIf CurrentLevel(0, RodentRow, intValueOne) = CAT_SPRITE Then
            Call KillRodent
            Exit Function
        End If
    
        'Start search for ground (Left or Right) (Must be pushing bushes)
        For i = intValueOne To intValueTwo Step IntStepVal
            'Make sure we don't push bushes over some other object like a cat'n'stuff...
            If CurrentLevel(0, RodentRow, i) = CAT_SPRITE Or CurrentLevel(0, RodentRow, i) = WALL_SPRITE Then
                Exit Function
                
            ElseIf CurrentLevel(0, RodentRow, i) = GROUND_SPRITE Or CurrentLevel(0, RodentRow, i) = CHEESE_SPRITE Then
                'Ground was found replace it with a bush...
                frmGameTable.picGameTable.PaintPicture frmGameTable.imgSprite(BUSH_SPRITE).Picture, i * TILE_WIDTH, RodentRow * TILE_HEIGHT, TILE_WIDTH, TILE_HEIGHT
    
                'Update the CurrentLevel array regarding the moved bush...
                CurrentLevel(0, RodentRow, RodentCol + IntStepVal) = GROUND_SPRITE 'Rid old data (Old bush Position)
                CurrentLevel(0, RodentRow, i) = BUSH_SPRITE 'New data
    
                'Move the rodent
                Call MoveRodent(RodentRow, RodentCol + IntStepVal, RodentRow, RodentCol)
    
                'Remember new position
                RodentCol = RodentCol + IntStepVal
                Exit For
            End If
        Next
    
    ElseIf intKeyCode = vbKeyUp Or intKeyCode = vbKeyDown Then
        'Then we should check the ground right in front (North or South) of the
        'rodent for ground...
        If CurrentLevel(0, intValueOne, RodentCol) = GROUND_SPRITE Then
            'Well because there is ground right in front of us we can skip
            'the rest of this function and place the rodent there BUT first
            'erase the rodents previous location.
            Call MoveRodent(intValueOne, RodentCol, RodentRow, RodentCol)
            
            'Remember new position
            RodentRow = intValueOne
            Exit Function

        ElseIf CurrentLevel(0, intValueOne, RodentCol) = CHEESE_SPRITE Then
            'Hey the player ate some cheese, give some points!!
            Call UpdateScore(CHEESE_EATING_POINTS)
            
            Call MoveRodent(intValueOne, RodentCol, RodentRow, RodentCol)
            
            'Remember new position
            RodentRow = intValueOne
            Exit Function

        ElseIf CurrentLevel(0, intValueOne, RodentCol) = CAT_SPRITE Then
            Call KillRodent
            Exit Function
        End If
    
        'Start search for ground (Up or Down)
        For i = intValueOne To intValueTwo Step IntStepVal
            'Make sure we don't push bushes over some other object like a cat'n'stuff...
            If CurrentLevel(0, i, RodentCol) = CAT_SPRITE Or CurrentLevel(0, i, RodentCol) = WALL_SPRITE Then
                Exit Function
                
            ElseIf CurrentLevel(0, i, RodentCol) = GROUND_SPRITE Or CurrentLevel(0, i, RodentCol) = CHEESE_SPRITE Then
                'Ground was found replace it with a bush...
                frmGameTable.picGameTable.PaintPicture frmGameTable.imgSprite(BUSH_SPRITE).Picture, RodentCol * TILE_WIDTH, i * TILE_HEIGHT, TILE_WIDTH, TILE_HEIGHT

                'Update the CurrentLevel array regarding the moved bush...
                CurrentLevel(0, RodentRow + IntStepVal, RodentCol) = GROUND_SPRITE 'Rid old data (Old bush Position)
                CurrentLevel(0, i, RodentCol) = BUSH_SPRITE 'New data

                'Move the rodent
                Call MoveRodent(RodentRow + IntStepVal, RodentCol, RodentRow, RodentCol)

                'Remember new position
                RodentRow = RodentRow + IntStepVal
                Exit Function
            End If
        Next
    End If

End Function

'-----------------------------------------------------------
Function MoveRodent(intRow As Integer, intCol As Integer, intOldRow As Integer, intOldCol As Integer)
'This function moves the rodent and updates it's position in the
'CurrentLevel array.
'intRow     = New Rodent Row
'intCol     = New Rodent Col
'intOldRow  = Old Rodent Row (Used to erase old position)
'intOldCol  = Old Rodent Col (Used to erase old position)
    
    'Place a ground tile at his current position...
    frmGameTable.picGameTable.PaintPicture frmGameTable.imgSprite(GROUND_SPRITE).Picture, intOldCol * TILE_WIDTH, intOldRow * TILE_HEIGHT, TILE_WIDTH, TILE_HEIGHT
    
    'Place the rodent at the new location...
    frmGameTable.picGameTable.PaintPicture frmGameTable.imgSprite(RODENT_SPRITE).Picture, intCol * TILE_WIDTH, intRow * TILE_HEIGHT, TILE_WIDTH, TILE_HEIGHT
    
    'Update the CurrentLevel array regarding the moved rodent...
    CurrentLevel(0, intOldRow, intOldCol) = GROUND_SPRITE 'Rid old data (Old rodent Position)
    CurrentLevel(0, intRow, intCol) = RODENT_SPRITE 'New data
    
End Function

'-----------------------------------------------------------
Function MoveCat(CatIndex As Integer, CatNewRow As Integer, CatNewCol As Integer, ShouldSleep As Boolean)
'This function moves the cat and updates it's position in the level array.

    'Remove cat from screen and array...
    frmGameTable.picGameTable.PaintPicture frmGameTable.imgSprite(GROUND_SPRITE).Picture, Cats(CatIndex).Col * TILE_WIDTH, Cats(CatIndex).Row * TILE_HEIGHT
    CurrentLevel(0, Cats(CatIndex).Row, Cats(CatIndex).Col) = GROUND_SPRITE
    
    'Update cat array...
    Cats(CatIndex).Row = Cats(CatIndex).Row + CatNewRow
    Cats(CatIndex).Col = Cats(CatIndex).Col + CatNewCol
    If ShouldSleep = True Then
        Cats(CatIndex).State = CAT_SLEEPING
    Else
        Cats(CatIndex).State = CAT_ACTIVE
    End If
    
    'Update Level Data array...
    CurrentLevel(0, Cats(CatIndex).Row, Cats(CatIndex).Col) = CAT_SPRITE
    
    'Repaint Cat...
    If ShouldSleep = True Then
        frmGameTable.picGameTable.PaintPicture frmGameTable.imgSprite(CATSLEEPING_SPRITE).Picture, Cats(CatIndex).Col * TILE_WIDTH, Cats(CatIndex).Row * TILE_HEIGHT
    Else
        frmGameTable.picGameTable.PaintPicture frmGameTable.imgSprite(CAT_SPRITE).Picture, Cats(CatIndex).Col * TILE_WIDTH, Cats(CatIndex).Row * TILE_HEIGHT
    End If
End Function

'-----------------------------------------------------------
Function MoveCats()
'Moves AI cats into their new positions after
'collision checking.
    
    Dim i As Integer
    Dim RowDirection As Integer, ColDirection   As Integer
    Dim Row As Integer, RowVal1 As Integer, RowVal2 As Integer
    Dim Col As Integer, ColVal1 As Integer, ColVal2 As Integer
    Dim StepDirection1 As Integer, StepDirection2 As Integer
    Dim RowMovement As Integer, ColMovement As Integer
    
    'Check to see if the wave ended by checking to see
    'if all cats are asleep, if they are turn the to cheese
    'and start the next wave...
    For i = 0 To CatsInWave - 1 ' #021012 ' UBound(Cats)
        If Cats(i).State = CAT_ACTIVE Then
            GoTo SkipNewWave
        End If
    Next
    
    'Well we made it this far... I guess there are no active cats!
    Call BeginNewWave
    Exit Function
    
SkipNewWave:
    'Loop through all cats currently in the game and
    'move them...
    For i = LBound(Cats) To CatsInWave - 1 ' #021012 ' UBound(Cats)
        If Cats(i).State = CAT_ACTIVE Or Cats(i).State = CAT_SLEEPING Then
            'Find what direction the mouse is at so we can move closer...
            'Compare row positions...
            If Cats(i).Row < RodentRow Then
                'He's below us
                RowDirection = 1
                StepDirection1 = -1
            ElseIf Cats(i).Row > RodentRow Then
                'He's above us
                RowDirection = -1
                StepDirection1 = 1
            Else
                'He's on the same row
                RowDirection = 0
                StepDirection1 = -1
            End If
            
            'Compare column positions...
            If Cats(i).Col < RodentCol Then
                'He's on the right
                ColDirection = 1
                StepDirection2 = -1
            ElseIf Cats(i).Col > RodentCol Then
                'He's on the left
                ColDirection = -1
                StepDirection2 = 1
            Else
                'He's on the same col
                ColDirection = 0
                StepDirection2 = -1
            End If
            
            If StepDirection1 = -1 Then
                RowVal1 = 3
                RowVal2 = 1
            Else
                RowVal1 = 1
                RowVal2 = 3
            End If
            
            If StepDirection2 = -1 Then
                ColVal1 = 3
                ColVal2 = 1
            Else
                ColVal1 = 1
                ColVal2 = 3
            End If
            
            'Now that we know what direction the mouse is in, we need to
            'check to see if we can move in the direction and if so to it,
            'however if we can't, make the cat go to sleep and exit the
            'function...
            For Row = RowVal1 To RowVal2 Step StepDirection1
                If RowDirection < -1 Then
                    RowDirection = 1
                End If
                
                For Col = ColVal1 To ColVal2 Step StepDirection2
                    If ColDirection < -1 Then
                        ColDirection = 1
                    End If
                    
                    If CurrentLevel(0, Cats(i).Row + RowDirection, Cats(i).Col + ColDirection) = GROUND_SPRITE Then
                        Call MoveCat(i, RowDirection, ColDirection, False)
                        GoTo NextCat
                    ElseIf CurrentLevel(0, Cats(i).Row + RowDirection, Cats(i).Col + ColDirection) = RODENT_SPRITE Then
                        'Do death animation...
                        Call KillRodent
                        
                        Call MoveCat(i, RowDirection, ColDirection, False)
                        GoTo NextCat
                    Else
                        ColDirection = ColDirection - 1
                    End If
                Next
                
                RowDirection = RowDirection - 1
            Next
            
            'Well if we made it this far the cat must be trapped so let
            'the cat go to sleep and exit the function...
            Call MoveCat(i, 0, 0, True)
        End If
        
NextCat:
    Next
End Function

'-----------------------------------------------------------
Function KillRodent()
    'Make sure the arrow keys know that we are
    'in the process of dieing...
    DieSceneActive = True
    
    'Stop cats from moving...
    frmGameTable.tmrCatTimer.Enabled = False
    
    'Start death scene...
    frmGameTable.tmrDieAnimation.Enabled = True
    
End Function

'-----------------------------------------------------------
Function BeginNewWave()
'Loops through the cat array and turns all sleeping cats
'to cheese, then places the number of cats in the last wave
'plus one on the field. If the number of cats is more then 10
'a new level would be started.

    Dim i As Integer
    Dim CatsAdded As Integer
    Dim RandomNumber As Integer, RandomNumber2 As Integer
    
    'Do we have less than the default of ten cats on the
    'previous field? If not we have to start a new level.
    If CatsInWave + 1 > START_NEW_LEVEL_AFTER_WAVE Then  '  #021012
    
        CatsInWave = 1
        
        ' #02101
        LevelNumber = LevelNumber + 1
        Call InitializeVariables(LevelNumber, False)
        Call DrawGameTable
        Exit Function
        
    End If
    
    'Turn all sleeping cats to cheese...
    For i = 0 To CatsInWave - 1 ' #021012 ' UBound(Cats)
        If Cats(i).State = CAT_SLEEPING Then
            frmGameTable.picGameTable.PaintPicture frmGameTable.imgSprite(CHEESE_SPRITE).Picture, Cats(i).Col * TILE_WIDTH, Cats(i).Row * TILE_HEIGHT
            
            'Update Level Data array...
            CurrentLevel(0, Cats(i).Row, Cats(i).Col) = CHEESE_SPRITE
        End If
    Next
    
    'Add another cat to the wave...
    CatsInWave = CatsInWave + 1
    
    CatsAdded = 0 ' #021012 ' -1
    
    'Place new cats on field...
    Do Until CatsAdded >= CatsInWave
        Randomize
        RandomNumber = Int((NumTableRows * Rnd))
        Randomize
        RandomNumber2 = Int((NumTableCols * Rnd))
        
        If CurrentLevel(0, RandomNumber, RandomNumber2) = GROUND_SPRITE Then
            'Draw cat...
            frmGameTable.picGameTable.PaintPicture frmGameTable.imgSprite(CAT_SPRITE).Picture, RandomNumber2 * TILE_WIDTH, RandomNumber * TILE_HEIGHT
            
            'Update Level Data array...
            CurrentLevel(0, RandomNumber, RandomNumber2) = CAT_SPRITE
            
            
            'Update Cat Array
            With Cats(CatsAdded)
                .Col = RandomNumber2
                .Row = RandomNumber
                .State = CAT_ACTIVE
            End With
            
            
            CatsAdded = CatsAdded + 1   ' #021012 moved below "With" block.
            
        End If
    Loop
End Function

'-----------------------------------------------------------
Function UpdateScore(PointsToAdd As Integer)
'Updates the points on the UI.
    Score = Score + PointsToAdd
    frmGameTable.lblScore.Caption = "Score: " & Format(Score, "#,###,###")
End Function
